/***********************************************************************************************************************
 *
 * blueBill Stats
 * Copyright (C) 2011-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile/
 * SCM: http://java.net/hg/bluebill-server~stats-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.stats.domain.impl;

import javax.annotation.Nonnull;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Map;
import it.tidalwave.util.DefaultFilterSortCriterion;
import it.tidalwave.util.Finder.SortCriterion;
import it.tidalwave.util.spi.SimpleFinderSupport;
import it.tidalwave.bluebill.stats.domain.Ping;
import it.tidalwave.bluebill.stats.domain.PingFinder;

/***********************************************************************************************************************
 *
 * A support for {@link LogPingDao} finders.
 * 
 * @stereotype Finder
 * 
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
/* package */ abstract class HttpServerLogPingFinderSupport extends SimpleFinderSupport<Ping> implements PingFinder 
  {
    private static final SortCriterion BY_TIMESTAMP_IMPL = new DefaultFilterSortCriterion(new Comparator<Ping>() 
      {
        @Override
        public int compare (final @Nonnull Ping p1, final @Nonnull Ping p2) 
          {
            return p1.getTimestamp().compareTo(p2.getTimestamp());
          }
      }, "BY_TIMESTAMP");
    
    private static final SortCriterion BY_OS_VERSION_IMPL = new DefaultFilterSortCriterion(new Comparator<Ping>() 
      {
        @Override
        public int compare (final @Nonnull Ping p1, final @Nonnull Ping p2) 
          {
            return p1.getOsVersion().compareTo(p2.getOsVersion());
          }
      }, "BY_OS_VERSION");
    
    private static final SortCriterion BY_MODEL_IMPL = new DefaultFilterSortCriterion(new Comparator<Ping>() 
      {
        @Override
        public int compare (final @Nonnull Ping p1, final @Nonnull Ping p2) 
          {
            return p1.getModel().compareTo(p2.getModel());
          }
      }, "BY_MODEL");
    
    private static final SortCriterion BY_INSTALL_ID_IMPL = new DefaultFilterSortCriterion(new Comparator<Ping>() 
      {
        @Override
        public int compare (final @Nonnull Ping p1, final @Nonnull Ping p2) 
          {
            return p1.getInstallId().compareTo(p2.getInstallId());
          }
      }, "BY_INSTALL_ID");
    
    private static final Map<SortCriterion, SortCriterion> sortCriterionImplementationMap = new HashMap<SortCriterion, SortCriterion>()
      {{
        put(PingFinder.BY_TIMESTAMP, BY_TIMESTAMP_IMPL);
        put(PingFinder.BY_OS_VERSION, BY_OS_VERSION_IMPL);
        put(PingFinder.BY_MODEL, BY_MODEL_IMPL);
        put(PingFinder.BY_INSTALL_ID, BY_INSTALL_ID_IMPL);
      }};
    
    /*******************************************************************************************************************
     *
     * {@inheritDoc}
     * 
     ******************************************************************************************************************/
    @Override @Nonnull
    public PingFinder sort (final @Nonnull SortCriterion criterion,
                            final @Nonnull SortDirection direction)
      {
        final SortCriterion criterionImpl = sortCriterionImplementationMap.get(criterion);

        if (criterionImpl != null)
          {
            return (PingFinder)super.sort(criterionImpl, direction);
          }

        throw new UnsupportedOperationException("Unsupported sortCriterion: " + criterion 
                                                + " - supported are: " + sortCriterionImplementationMap.keySet()); 
      }
  }
